import base64
import hashlib
from Crypto import Random
from Crypto.Cipher import AES

# LINK Implementation of seedable AES encryption thanks to: https://stackoverflow.com/a/21928790
class AESCipher(object):

    def __init__(self, key):
        self.bs = AES.block_size
        self.key = hashlib.sha256(key.encode()).digest()

    def encrypt(self, raw):
        raw = self._pad(raw)
        iv = Random.new().read(AES.block_size)
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return base64.b64encode(iv + cipher.encrypt(raw.encode()))

    def decrypt(self, enc):
        enc = base64.b64decode(enc)
        iv = enc[:AES.block_size]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return AESCipher._unpad(cipher.decrypt(enc[AES.block_size:])).decode('utf-8')

    def _pad(self, s):
        return s + (self.bs - len(s) % self.bs) * chr(self.bs - len(s) % self.bs)

    @staticmethod
    def _unpad(s):
        return s[:-ord(s[len(s)-1:])]
    

# SECTION implementable methods
# NOTE The cipher is initialized on the fly, so the seed is not stored
    
def self_encrypt_aes(seed, message):
    cipher = AESCipher(seed)
    encrypted = cipher.encrypt(message)
    return encrypted

def self_decrypt_aes(seed, encrypted):
    cipher = AESCipher(seed)
    decrypted = cipher.decrypt(encrypted)
    return decrypted
